@@ -5,6 +5,7 @@
 # Returns ar array of references to associative arrays, each containing
 # information about some directive. The keys of each array are:
 #  name -	The name of this directive
+#  multiple -	Can this directive appear multiple times
 #  type -	What kind of directive this in. Possible values are
 #		0 - Processes and limits
 #		1 - Networking and addresses
@@ -27,7 +28,6 @@
 #		18- Filters
 #		19- Character Sets
 #		20- Image maps
-#  multiple -	Can this directive appear multiple times
 #  global -	Can be used in the global server context
 #  virtual -	Can be used in a VirtualHost section or in the global section
 #  directory -	Can be used in a Directory section context
@@ -37,7 +37,8 @@ sub core_directives
 local($rv);
 $rv = [	[ 'AccessFileName', 0, 5, 'virtual', undef, 5 ],
 	[ 'AddDefaultCharset', 0, 19, 'virtual directory htaccess', 2.0 ],
-	[ 'ClearModuleList AddModule', 1, 2, 'global', -2.0 ],
+	&can_configure_apache_modules() ? ( ) :
+		( [ 'ClearModuleList AddModule', 1, 2, 'global', -2.0 ] ),
 	[ 'AllowOverride', 0, 5, 'directory' ],
 	[ 'AuthName', 0, 4, 'directory htaccess', undef, 10 ],
 	[ 'AuthType', 0, 4, 'directory htaccess', undef, 8 ],
@@ -68,9 +69,10 @@ $rv = [	[ 'AccessFileName', 0, 5, 'virtual', undef, 5 ],
 	[ 'StartServers', 0, 0, 'global', -2.0 ],
 	[ 'MinSpareServers', 0, 0, 'global', -2.0 ],
 	[ 'MaxSpareServers', 0, 0, 'global', -2.0 ],
-	[ 'NameVirtualHost', 1, 1, 'global', 1.3, 5 ],
+	[ 'NameVirtualHost', 1, 1, 'global', '1.3-2.4', 5 ],
 	[ 'Options', 0, 5, 'virtual directory htaccess', undef, 3 ],
 	[ 'PidFile', 0, 9, 'global', -2.0 ],
+	[ 'Protocols', 0, 1, 'virtual', 2.4 ],
 	[ 'require', 0, 4, 'directory htaccess', undef, 6 ],
 	[ 'RLimitCPU', 0, 0, 'virtual', 1.2 ],
 	[ 'RLimitMEM', 0, 0, 'virtual', 1.2 ],
@@ -245,20 +247,33 @@ elsif ($in{"$_[0]_mode"} == 2) {
 # Networking and address directives
 sub edit_BindAddress_Listen_Port
 {
-local($bref, $lref, $pref, @blist, @plist, $inp);
+local($bref, $lref, $pref, @blist, @plist, @slist, $inp);
 $bref = $_[0]; $lref = $_[1]; $pref = $_[2];
 if (@$lref) {
 	# listen directives in use.. so BindAddress and Port are unused
 	foreach $l (@$lref) {
-		if ($l->{'value'} =~ /^(\S+):(\d+)$/) {
+		my @w = split(/\s+/, $l->{'value'});
+		if ($w[0] =~ /^\[(\S+)\]:(\d+)$/) {
+			# IPv6 address and port
 			push(@blist, $1); push(@plist, $2);
 			}
-		elsif ($l->{'value'} =~ /^(\d+)$/) {
-			push(@blist, "*"); push(@plist, $1);
-			}
-		elsif ($l->{'value'} =~ /^(\S+)$/) {
+		elsif ($w[0] =~ /^\[(\S+)\]$/) {
+			# IPv6 address only
 			push(@blist, $1); push(@plist, undef);
 			}
+		elsif ($w[0] =~ /^(\S+):(\d+)$/) {
+			# IPv4 address and port
+			push(@blist, $1); push(@plist, $2);
+			}
+		elsif ($w[0] =~ /^(\d+)$/) {
+			# Port only
+			push(@blist, "*"); push(@plist, $1);
+			}
+		elsif ($w[0] =~ /^(\S+)$/) {
+			# IPv4 address or hostname only
+			push(@blist, $1); push(@plist, undef);
+			}
+		push(@slist, $w[1]);
 		}
 	}
 else {
@@ -266,36 +281,48 @@ else {
 	if (@$bref) { push(@blist, $bref->[@$bref-1]->{'value'}); }
 	else { push(@blist, "*"); }
 	push(@plist, undef);
+	push(@slist, undef);
 	}
 $port = @$pref ? $pref->[@$pref-1]->{'value'} : 80;
 if ($_[3]->{'version'} < 2.0) {
-	$inp = "<b>$text{'core_dport'}</b> <input name=Port size=6 value=\"$port\"><br>\n";
+	$inp = "<b>$text{'core_dport'}</b> ".
+	       &ui_textbox("Port", $port, 6)."<br>\n";
 	}
-$inp .=	"<table border>\n".
-	"<tr $tb> <td><b>$text{'core_address'}</b></td> <td><b>$text{'core_port'}</b></td> </tr>\n";
+my @cols = ( $text{'core_address'}, $text{'core_port'} );
+if ($_[3]->{'version'} >= 2.4) {
+	# Apache supports a port protocol
+	push(@cols, $text{'core_portname'});
+	}
+$inp .= &ui_columns_start(\@cols, "50%");
 for($i=0; $i<@blist+1; $i++) {
-	$inp .= sprintf
-		"<tr><td><input type=radio name=BindAddress_def_$i value=2 %s>".
-		" $text{'core_none'} <input type=radio name=BindAddress_def_$i value=1 %s>".
-		" $text{'core_all'} <input type=radio name=BindAddress_def_$i value=0 %s> ".
-		"<input name=BindAddress_$i size=20 value=\"%s\"></td>",
-		$blist[$i] ? "" : "checked",
-		$blist[$i] eq "*" ? "checked" : "",
-		$blist[$i] && $blist[$i] ne "*" ? "checked" : "",
-		$blist[$i] eq "*" ? "" : $blist[$i];
+	my @row;
+	my $ba = $blist[$i] eq "*" ? 1 : $blist[$i] eq "" ? 2 : 0;
+	push(@row, &ui_radio("BindAddress_def_$i", $ba,
+			[ [ 2, $text{'core_none'} ],
+			  [ 1, $text{'core_all'} ],
+			  [ 0, &ui_textbox("BindAddress_$i",
+				$ba == 0 ? $blist[$i] : "", 20) ] ]));
 	if ($_[3]->{'version'} < 2.0) {
-		$inp .= "<td>".&opt_input($plist[$i], "Port_$i", "$text{'core_default'}", 5)."</td>";
+		push(@row, &opt_input($plist[$i], "Port_$i",
+				      $text{'core_default'}, 5));
 		}
 	else {
-		$inp .= "<td><input name=Port_$i size=5 value='$plist[$i]'></td>\n";
+		push(@row, &ui_textbox("Port_$i", $plist[$i], 5));
 		}
+	if ($_[3]->{'version'} >= 2.4) {
+		push(@row, &ui_select("Name_$i", $slist[$i],
+				      [ [ "", $text{'core_protoany'} ],
+					[ "http", "HTTP" ],
+					[ "https", "HTTPS" ] ]));
+		}
+	$inp .= &ui_columns_row(\@row);
 	}
-$inp .= "</table>\n";
+$inp .= &ui_columns_end();
 return (2, $text{'core_listen'}, $inp);
 }
 sub save_BindAddress_Listen_Port
 {
-local(@blist, @plist, $bdef, $b, $p);
+local(@blist, @plist, @slist, $bdef, $b, $p);
 
 # build list of addresses and ports
 for($i=0; defined($in{"Port_$i"}); $i++) {
@@ -304,12 +331,15 @@ for($i=0; defined($in{"Port_$i"}); $i++) {
 	if ($bdef == 2) { next; }
 
 	if ($bdef) { push(@blist, "*"); }
-	elsif ($b =~ /^\S+$/ && gethostbyname($b)) { push(@blist, $b); }
+	elsif ($b =~ /^\S+$/ &&
+	       (&to_ipaddress($b) || &to_ip6address($b))) { push(@blist, $b); }
 	else { &error(&text('core_eaddress', $b)); }
 
 	if ($pdef) { push(@plist, undef); }
 	elsif ($p =~ /^\d+$/) { push(@plist, $p); }
 	else { &error(&text('core_eport', $p)); }
+
+	push(@slist, $in{"Name_$i"});
 	}
 if (!@blist) { &error($text{'core_eoneaddr'}); }
 
@@ -337,12 +367,23 @@ if ($_[0]->{'version'} < 2.0) {
 	}
 else {
 	# Apache 2.0 just uses Listen directives
+	local %doneport;
 	for($i=0; $i<@blist; $i++) {
+		if (&check_ip6address($blist[$i])) {
+			$blist[$i] = "[".$blist[$i]."]";
+			}
 		if ($blist[$i] ne "*" && $plist[$i]) {
 			push(@l, "$blist[$i]:$plist[$i]");
 			}
 		elsif ($blist[$i] ne "*") { push(@l, $blist[$i]); }
 		else { push(@l, "*:$plist[$i]"); }
+		if ($doneport{$l[$#l]}++) {
+			# Same listen given twice
+			&error(&text('core_eduplisten', $l[$#l]));
+			}
+		if ($_[0]->{'version'} >= 2.4 && $slist[$i]) {
+			$l[$#l] .= " ".$slist[$i];
+			}
 		}
 	return ( [], \@l );
 	}
@@ -538,6 +579,7 @@ sub edit_NameVirtualHost
 local(@nv, $nv, $star);
 foreach $nv (@{$_[0]}) {
 	if ($nv->{'value'} eq "*" && $_[1]->{'version'} >= 1.312) { $star++; }
+	elsif ($nv->{'value'} =~ /^\[(\S+)\]$/) { push(@nv, $1); }
 	else { push(@nv, $nv->{'value'}); }
 	}
 if ($_[1]->{'version'} >= 1.312) {
@@ -556,16 +598,51 @@ local(@nv, $nv, $addr);
 @nv = split(/\s+/, $in{'NameVirtualHost'});
 @nv = ( "*", @nv ) if ($in{'NameVirtualHost_star'});
 foreach $nv (@nv) {
-	if ($nv =~ /^(\S+):(\d+|\*)$/) { $addr = $1; }
+	if ($nv =~ /^\[(\S+)\]:(\d+|\*)$/) { $addr = $1; }
+	elsif ($nv =~ /^(\S+):(\d+|\*)$/) { $addr = $1; }
 	else { $addr = $nv; }
-	if (!gethostbyname($addr) && $addr ne '*') {
+	if (!&to_ipaddress($addr) &&
+	    !&to_ip6address($addr) && $addr ne '*') {
 		&error(&text('core_evirtaddr', $addr));
 		}
+	if ($nv =~ /^(\S+):(\d+|\*)$/ && &check_ip6address($1)) {
+		$nv = "[$1]:$2";
+		}
+	elsif (&check_ip6address($nv)) {
+		$nv = "[$nv]";
+		}
 	}
 if (@nv) { return ( \@nv ); }
 else { return ( [ ] ); }
 }
 
+sub edit_Protocols
+{
+my %p = map { $_, 1 } @{$_[0]->{'words'}};
+my @popts = ( "h2", "h2c", "http/1.1" );
+my @psel = map { &ui_checkbox("protocols", $_, $text{'core_protocols_'.$_} || $_, $p{$_}) } @popts;
+return (1,
+        $text{'core_protocols'},
+	&ui_radio("protocols_def", %p ? 0 : 1,
+		  [ [ 1, $text{'default'}.
+		      " (".$text{'core_protocols_http/1.1'}.")<br>" ],
+		    [ 0, $text{'core_protocols_sel'} ] ])." ".
+	join(" ", @psel),
+	);
+}
+sub save_Protocols
+{
+if ($in{'protocols_def'}) {
+	return ( [ ] );
+	}
+else {
+	my @prots = split(/\0/, $in{'protocols'});
+	@prots || &error($text{'core_eprotocols'});
+	@prots = sort @prots;
+	return ( [ join(" ", @prots) ] );
+	}
+}
+
 #########################################################################
 # Document directives
 sub edit_AccessFileName
@@ -643,8 +720,12 @@ return (2, $text{'core_docroot'},
 }
 sub save_DocumentRoot
 {
-$in{'DocumentRoot_def'} || -d $in{'DocumentRoot'} ||
-	&error(&text('core_enodoc', $in{'DocumentRoot'}));
+if (!$in{'DocumentRoot_def'}) {
+	-d $in{'DocumentRoot'} ||
+		&error(&text('core_enodoc', $in{'DocumentRoot'}));
+	&allowed_doc_dir($in{'DocumentRoot'}) ||
+		&error(&text('core_ecandoc', $in{'DocumentRoot'}));
+	}
 return &parse_opt("DocumentRoot");
 }
 
@@ -698,11 +779,23 @@ local(@po, $po, @rv);
 if ($in{'Options_def'}) { return ( [ ] ); }
 @po = ("ExecCGI", "FollowSymLinks", "Includes", "IncludesNOEXEC",
        "Indexes", "MultiViews", "SymLinksIfOwnerMatch");
+my $abscount = 0;
+my $pluscount = 0;
 foreach $po (@po) {
-	if ($in{$po} == 1) { push(@rv, $po); }
-	elsif ($in{$po} == 2) { push(@rv, "+$po"); }
-	elsif ($in{$po} == 3) { push(@rv, "-$po"); }
+	if ($in{$po} == 1) {
+		push(@rv, $po);
+		$abscount++;
+		}
+	elsif ($in{$po} == 2) {
+		push(@rv, "+$po");
+		$pluscount++;
+		}
+	elsif ($in{$po} == 3) {
+		push(@rv, "-$po");
+		$pluscount++;
+		}
 	}
+$abscount && $pluscount && &error($text{'core_eoptionsboth'});
 return @rv ? ( [ join(' ', @rv) ] ) : ( [ "None" ] );
 }
 
@@ -784,8 +877,8 @@ return &parse_opt("ForceType", '^\S+\/\S+$', $text{'mod_mime_etype'});
 sub edit_SetOutputFilter
 {
 local @vals = split(/[\s;]+/, $_[0]->{'value'});
-return (2, $text{'core_outfilter'},
-	&filters_input(\@vals, "SetOutputFilter"));
+my $fi = &filters_input(\@vals, "SetOutputFilter");
+return (2, $text{'core_outfilter'}, $fi);
 }
 sub save_SetOutputFilter
 {
@@ -795,8 +888,8 @@ return &parse_filters("SetOutputFilter");
 sub edit_SetInputFilter
 {
 local @vals = split(/[\s;]+/, $_[0]->{'value'});
-return (2, $text{'core_infilter'},
-	&filters_input(\@vals, "SetInputFilter"));
+my $fi = &filters_input(\@vals, "SetInputFilter");
+return (2, $text{'core_infilter'}, $fi);
 }
 sub save_SetInputFilter
 {
@@ -853,7 +946,7 @@ sub edit_AuthType
 local($rv, $a);
 $rv = "<select name=AuthType>\n";
 foreach $a ("", "Basic", "Digest") {
-	$rv .= sprintf "<option %s>$a\n",
+	$rv .= sprintf "<option %s>$a</option>\n",
 	        lc($_[0]->{'value'}) eq lc($a) ? "selected" : "";
 	}
 $rv .= "</select>";
@@ -868,7 +961,9 @@ else { return ( [ ] ); }
 sub edit_require
 {
 local($rv, $mode, $list);
-$_[0]->{'value'} =~ /^(\S+)\s*(.*)$/; $mode = $1; $list = $2;
+local @w = @{$_[0]->{'words'}};
+$mode = shift(@w);
+$list = join(" ", map { $_ =~ /\s/ ? "\"$_\"" : $_ } @w);
 
 # All users
 $rv = sprintf
@@ -881,7 +976,7 @@ $rv .= sprintf
       $mode eq "user" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_user size=20 value=\"%s\"><br>\n",
-      $mode eq "user" ? $list : "";
+      $mode eq "user" ? &html_escape($list) : "";
 
 # Only members of groups
 $rv .= sprintf
@@ -889,7 +984,7 @@ $rv .= sprintf
       $mode eq "group" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_group size=20 value=\"%s\"><br>\n",
-      $mode eq "group" ? $list : "";
+      $mode eq "group" ? &html_escape($list) : "";
 
 # All users
 $rv .= sprintf
@@ -958,7 +1053,7 @@ sub edit_LimitRequestBody
 {
 return (1, $text{'core_maxbody'},
 	&opt_input($_[0]->{'value'}, "LimitRequestBody", $text{'core_default'}, 8)
-		.$text{'bytes'});
+		.&ui_space(2).$text{'bytes'});
 }
 sub save_LimitRequestBody
 {
@@ -969,7 +1064,7 @@ sub edit_LimitXMLRequestBody
 {
 return (1, $text{'core_maxxml'},
 	&opt_input($_[0]->{'value'}, "LimitXMLRequestBody",
-		   $text{'core_default'}, 8).$text{'bytes'});
+		   $text{'core_default'}, 8).&ui_space(2).$text{'bytes'});
 }
 sub save_LimitXMLRequestBody
 {
@@ -1045,10 +1140,13 @@ sub edit_ServerTokens
 {
 local $v = $_[0]->{'value'};
 $v = "ProductOnly" if ($v eq "Prod");
+$v = "Min" if ($v eq "Minimal");
 return (1, $text{'core_header'},
 	&select_input($v, "ServerTokens", "Full",
 		      "$text{'core_verosmod'},Full",
-		      "$text{'core_veros'},OS", "$text{'core_ver'},Min",
+		      "$text{'core_veros'},OS",
+		      "$text{'core_ver'},Min",
+		      "$text{'core_minor'},Minor",
 		      $_[1]->{'version'} >= 1.313 ?
 			("$text{'core_product'},ProductOnly") : (),
 		      $_[1]->{'version'} >= 2.041 ?
@@ -1125,7 +1223,7 @@ $rv .= "<tr $tb> <td><b>$text{'core_error'}</b></td> <td><b>$text{'core_resp'}</
 $len = @{$_[0]} + 1;
 for($i=0; $i<$len; $i++) {
 	$v = $_[0]->[$i]->{'value'};
-	if ($v =~ /^(\d+)\s+((http|ftp|gopher):\S+)$/)
+	if ($v =~ /^(\d+)\s+((http|https|ftp|gopher):\S+)$/)
 		{ $code = $1; $type = 0; $url = $2; }
 	elsif ($v =~ /^(\d+)\s+(\/.*)$/) { $code = $1; $type = 0; $url = $2; }
 	elsif ($v =~ /^(\d+)\s+"(.*)"$/) { $code = $1; $type = 1; $url = $2; }
@@ -1247,6 +1345,8 @@ return &parse_select("LogLevel", "");
 
 #########################################################################
 # Module directives
+# This isn't shown if the distro has a way of managing these, such as Debian's
+# /etc/apache/mods-enabled
 sub edit_ClearModuleList_AddModule
 {
 local($mods, @allmods, $d, %mods, $m, $i, $rv);
