@@ -1,6 +1,8 @@
 # mailbox-lib.pl
 use strict;
 use warnings;
+no warnings 'redefine';
+no warnings 'uninitialized';
 our (%text, %config, %gconfig, %userconfig);
 our $remote_user;
 our @remote_user_info;
@@ -162,28 +164,30 @@ sub list_addresses
 {
 my @rv;
 my $i = 0;
-open(my $ADDRESS, "<", $address_book);
-while(<$ADDRESS>) {
-	s/\r|\n//g;
-	my @sp = split(/\t/, $_);
-	if (@sp >= 1) {
-		push(@rv, [ $sp[0], $sp[1], $i, $sp[2] ]);
-		}
-	$i++;
-	}
-close($ADDRESS);
-if ($config{'global_address'}) {
-	my $gab = &group_subs($config{'global_address'});
-	open(my $ADDRESS, "<", $gab);
+if (open(my $ADDRESS, "<", $address_book)) {
 	while(<$ADDRESS>) {
 		s/\r|\n//g;
-		my @sp = split(/\t+/, $_);
-		if (@sp >= 2) {
-			push(@rv, [ $sp[0], $sp[1] ]);
+		my @sp = split(/\t/, $_);
+		if (@sp >= 1) {
+			push(@rv, [ $sp[0], $sp[1], $i, $sp[2] ]);
 			}
+		$i++;
 		}
 	close($ADDRESS);
 	}
+if ($config{'global_address'}) {
+	my $gab = &group_subs($config{'global_address'});
+	if (open(my $ADDRESS, "<", $gab)) {
+		while(<$ADDRESS>) {
+			s/\r|\n//g;
+			my @sp = split(/\t+/, $_);
+			if (@sp >= 2) {
+				push(@rv, [ $sp[0], $sp[1] ]);
+				}
+			}
+		close($ADDRESS);
+		}
+	}
 if ($userconfig{'sort_addrs'} == 2) {
 	return sort { lc($a->[0]) cmp lc($b->[0]) } @rv;
 	}
@@ -254,9 +258,11 @@ if (@list_folders_cache) {
 my (@rv, $f, $o, %done);
 
 # Read the module's config directory, to find folders files
-opendir(DIR, $user_module_config_directory);
-my @folderfiles = readdir(DIR);
-closedir(DIR);
+my @folderfiles;
+if (opendir(DIR, $user_module_config_directory)) {
+	@folderfiles = readdir(DIR);
+	closedir(DIR);
+	}
 
 if ($config{'mail_system'} == 2) {
 	# POP3 inbox
@@ -296,6 +302,11 @@ elsif ($config{'mail_system'} == 4) {
 		$rv[0]->{'autouser'} = 1;
 		}
 
+	# No need to prompt for a login if using the local dovecot command
+	if ($imapserver eq "*") {
+		$rv[0]->{'autouser'} = 1;
+		}
+
 	# Get other IMAP folders (if we can)
 	my ($ok, $ih) = &imap_login($rv[0]);
 	if ($ok == 1) {
@@ -313,6 +324,7 @@ elsif ($config{'mail_system'} == 4) {
 					    'server' => $imapserver,
 					    'user' => $rv[0]->{'user'},
 					    'pass' => $rv[0]->{'pass'},
+					    'autouser' => $rv[0]->{'autouser'},
 					    'mode' => 0,
 					    'remote' => 1,
 					    'flags' => 1,
@@ -331,10 +343,12 @@ elsif ($config{'mail_system'} == 4) {
 	my $sf;
 	my $sent;
 	if ($userconfig{'sent_name'}) {
+		$sf = $userconfig{'sent_name'};
 		($sent) = grep { lc($_->{'name'}) eq lc($sf) } @rv;
 		}
 	else {
-		($sent) = grep { lc($_->{'name'}) eq 'sent' } @rv;
+		$sf = "sent";
+		($sent) = grep { lc($_->{'name'}) eq $sf } @rv;
 		if (!$sent) {
 			($sent) = grep { $_->{'name'} =~ /sent/i } @rv;
 			}
@@ -638,7 +652,7 @@ foreach my $f (@folderfiles) {
 		}
 	}
 
-# When in IMAP inbox mode, we goto this label to skip all my folders
+# When in IMAP inbox mode, we goto this label to skip all local folders
 IMAPONLY:
 
 # Add user-defined composite folders
@@ -662,42 +676,41 @@ foreach my $f (@folderfiles) {
 
 # Add spam folder as specified in spamassassin module, in case it is
 # outside of the folders we scan
-if (&foreign_check("spam")) {
-	my %suserconfig = &foreign_config("spam", 1);
-	my $file = $suserconfig{'spam_file'};
-	$file ||= "";
+my %suserconfig;
+if ($config{'mail_system'} == 4) {
+	# In IMAP mode, the first folder named spam is marked
+	my ($sf) = grep { lc($_->{'name'}) eq 'spam' } @rv;
+	if ($sf) {
+		$sf->{'name'} = $text{'folder_spam'};
+		$sf->{'spam'} = 1;
+		}
+	}
+elsif (&foreign_check("spam") &&
+       (%suserconfig = &foreign_config("spam", 1)) &&
+       (my $file = $suserconfig{'spam_file'})) {
 	$file =~ s/\.$//;
 	$file =~ s/\/$//;
 	$file = "$remote_user_info[7]/$file" if ($file && $file !~ /^\//);
 	$file =~ s/\~/$remote_user_info[7]/;
-	if ($file) {
-		if ($config{'mail_system'} == 4) {
-			# In IMAP mode, the first folder named spam is marked
-			my ($sf) = grep { lc($_->{'name'}) eq 'spam' } @rv;
-			if ($sf) {
-				$sf->{'spam'} = 1;
-				}
-			}
-		elsif (!$done{$file}) {
-			# Need to add
-			push(@rv, { 'name' => "Spam",
-				    'file' => $file,
-				    'type' => &folder_type($file),
-				    'perpage' => $userconfig{"perpage_$file"},
-				    'fromaddr' => $userconfig{"fromaddr_$file"},
-				    'sent' => $userconfig{"sent_$file"},
-				    'hide' => 0,
-				    'mode' => 0,
-				    'spam' => 1,
-				    'index' => scalar(@rv) } );
-			$done{$file}++;
-			}
-		else {
-			# Mark as spam folder
-			my ($sf) = grep { $_->{'file'} eq $file } @rv;
-			if ($sf) {
-				$sf->{'spam'} = 1;
-				}
+	if (!$done{$file}) {
+		# Need to add
+		push(@rv, { 'name' => "Spam",
+			    'file' => $file,
+			    'type' => &folder_type($file),
+			    'perpage' => $userconfig{"perpage_$file"},
+			    'fromaddr' => $userconfig{"fromaddr_$file"},
+			    'sent' => $userconfig{"sent_$file"},
+			    'hide' => 0,
+			    'mode' => 0,
+			    'spam' => 1,
+			    'index' => scalar(@rv) } );
+		$done{$file}++;
+		}
+	else {
+		# Mark as spam folder
+		my ($sf) = grep { $_->{'file'} eq $file } @rv;
+		if ($sf) {
+			$sf->{'spam'} = 1;
 			}
 		}
 	}
@@ -758,6 +771,22 @@ foreach my $f (@rv) {
 		}
 	}
 
+# Filter out duplicate folders by inode
+my @frv;
+my %idone;
+foreach my $f (@rv) {
+	if (!$f->{'file'}) {
+		push(@frv, $f);
+		}
+	else {
+		my @st = stat($f->{'file'});
+		if (!@st || !$done{$st[0],$st[1]}++) {
+			push(@frv, $f);
+			}
+		}
+	}
+@rv = @frv;
+
 @list_folders_cache = @rv;
 return @rv;
 }
@@ -865,9 +894,11 @@ elsif ($folder->{'type'} == 6) {
 		}
 	my $i;
 	my $mems = $folder->{'members'};
-	for($i=0; $i<@$mems; $i++) {
-		$virt{$i} = &folder_name($mems->[$i]->[0])."\t".
-			    $mems->[$i]->[1];
+	if ($mems) {
+		for($i=0; $i<@$mems; $i++) {
+			$virt{$i} = &folder_name($mems->[$i]->[0])."\t".
+				    $mems->[$i]->[1];
+			}
 		}
 	&write_file("$user_module_config_directory/$folder->{'id'}.virt",
 		    \%virt);
@@ -1050,7 +1081,7 @@ if (@list_folders_cache) {
 # Delete mbox or Maildir index
 if ($folder->{'type'} == 0) {
 	my $ifile = &user_index_file($folder->{'file'});
-	unlink(glob("$ifile.*"), $ifile);
+	unlink(glob("\Q$ifile\E.*"), $ifile);
 	}
 elsif ($folder->{'type'} == 1) {
 	my $cachefile = &get_maildir_cachefile($folder->{'file'});
@@ -1059,7 +1090,7 @@ elsif ($folder->{'type'} == 1) {
 
 # Delete sort index
 my $ifile = &folder_new_sort_index_file($folder);
-unlink(glob("$ifile.*"), $ifile);
+unlink(glob("\Q$ifile\E.*"), $ifile);
 
 # Delete sort direction file
 my $file = &folder_name($folder);
@@ -2025,6 +2056,7 @@ sub get_sort_field
 {
 my ($folder) = @_;
 return ( ) if (!$folder->{'sortable'});
+return ( ) if (!$userconfig{'show_sort'});
 my $file = &folder_name($folder);
 $file =~ s/\//_/g;
 my %sort;
@@ -2049,6 +2081,7 @@ sub field_sort_link
 {
 my ($title, $field, $folder, $start) = @_;
 my ($sortfield, $sortdir) = &get_sort_field($folder);
+$sortfield ||= "";
 my $dir = $sortfield eq $field ? !$sortdir : 0;
 my $img = $sortfield eq $field && $dir ? "sortascgrey.gif" :
 	     $sortfield eq $field && !$dir ? "sortdescgrey.gif" :
@@ -2065,8 +2098,8 @@ else {
 sub view_mail_link
 {
 my ($folder, $id, $start, $txt) = @_;
-my $qid = &urlize($id);
-my $qstart = &urlize($start);
+my $qid = $id ? &urlize($id) : "";
+my $qstart = $start ? &urlize($start) : "";
 my $url = "view_mail.cgi?start=$qstart&id=$qid&folder=$folder->{'index'}";
 if ($userconfig{'open_mode'}) {
 	return "<a href='' onClick='window.open(\"$url\", \"viewmail\", \"toolbar=no,menubar=no,scrollbars=yes,width=1024,height=768\"); return false'>".
